!***********************************************************************
!
!   Copyright (C) 2010  Bill Paxton
!1;2802;0c
!   this file is part of mesa.
!
!   mesa is free software; you can redistribute it and/or modify
!   it under the terms of the gnu general library public license as published
!   by the free software foundation; either version 2 of the license, or
!   (at your option) any later version.
!
!   mesa is distributed in the hope that it will be useful, 
!   but without any warranty; without even the implied warranty of
!   merchantability or fitness for a particular purpose.  see the
!   gnu library general public license for more details.
!
!   you should have received a copy of the gnu library general public license
!   along with this software; if not, write to the free software
!   foundation, inc., 59 temple place, suite 330, boston, ma 02111-1307 usa
!
! ***********************************************************************
 
      module run_star_extras

      use star_lib
      use star_def
      use const_def
      
      implicit none
      integer time0, time1, clock_rate !variables defined for the timing
      ! these routines are called by the standard run_star check_model
      contains
      !Dutch mass loss algorithm, with tanh interpolation between the two Vink formulae
     subroutine  Dutch_tanh_intrp_wind(id, Lsurf, Msurf, Rsurf, Tsurf, w, ierr)                                                                  
         use crlibm_lib                                                                                                                    
         type (star_info), pointer :: s                                                                                                                      
         integer, intent(in) :: id                                                                                                                           
         real(dp), intent(in) :: Lsurf, Msurf, Rsurf, Tsurf ! surface values (cgs)                                                                           
         real(dp), intent(out) :: w ! wind in units of Msun/year (value is >= 0)                                                                             
         integer, intent(out) :: ierr                                                                                                                        
         real(dp) ::  L1, M1, R1, T1, xsurf, etaWR, Zsolar, Y, Z
         real(dp) :: log10w, w1, w2, T_high, T_low, alfa                                                                                            
         call get_star_ptr(id,s,ierr)                                                                                                                
         w = 0                                                                                                                                       
         ierr = 0                                                                                                                                    
         L1 = Lsurf                                                                                                                                  
         M1 = Msurf                                                                                                                                  
         R1 = Rsurf                                                                                                                                  
         T1 = Tsurf                                                                                                                                  
         Zsolar = 0.019         
         etaWR = s% Dutch_wind_eta 
         !Assumes the use of approx21.net                                                                                                            
         xsurf = s% xa(2,1)     !H mass fraction of the outermost cell, species 1 being neutrons                                                         
         Y = s% xa(5,1) + s% xa(4,1)                                                                 
         Z = 1 - Y -xsurf          
         T_high = 11000d0                                                                                                                              
         T_low = 10000d0                                                                                                                               
         if (xsurf .GE. 0.4d0) then                                                                                                            
            if (T1 <= T_low) then      
                print *, 'using: de_Jager, eta=', s% Dutch_wind_eta
                call eval_de_Jager_wind(w)                                                                                                               
            else if (T1 >= T_high) then                            
                print *, 'using: Vink_tanh, eta=', s% Dutch_wind_eta
                call eval_tanh_intrp_vink_wind(w)                                                                                                            
            else               ! transition
               print *, 'interpolating de Jager and Vink_tanh, eta=', s% Dutch_wind_eta                                                             
                call eval_de_Jager_wind(w1)                                                                                                                 
                call eval_tanh_intrp_vink_wind(w2)                                                                                                
                alfa = (T1 - T_low)/(T_high - T_low)                                                                                                     
                w = (1-alfa)*w1 + alfa*w2                                                                                                             
             end if 
         else !means it's a WR star
            print *, 'using: N&L, eta=', s% Dutch_wind_eta            
            w = 1d-11 * pow_cr(L1/Lsun,1.29d0) * pow_cr(Y,1.7d0) * sqrt(Z) 
         end if 
         w = s% Dutch_wind_eta * w                   
         
         contains

         subroutine eval_de_Jager_wind(w)
        ! de Jager, C., Nieuwenhuijzen, H., & van der Hucht, K. A. 1988, A&AS, 72, 259.                                             
         real(dp), intent(out) :: w
            real(dp) :: log10w
            include 'formats'
            log10w = 1.769d0*log10_cr(L1/Lsun) - 1.676d0*log10_cr(T1) - 8.158d0                                                                              
            w = exp10_cr(log10w)
         end subroutine eval_de_Jager_wind

      subroutine eval_tanh_intrp_vink_wind(w)
          real(dp), intent(inout) :: w
          real(dp) :: alfa, w1, w2, w_c, w_h, Teff_jump, logMdot, dT, vinf_div_vesc   
          w = 0
          
         if (T1 > 27500d0) then
             vinf_div_vesc = 2.6d0 ! this is the hot side galactic value                                                                                    
             vinf_div_vesc = vinf_div_vesc*pow_cr(Z/Zsolar,0.13d0) ! corrected for Z                                                                        
              logMdot = &
                  - 6.697d0 &
                  + 2.194d0*log10_cr(L1/Lsun/1d5) &
                  - 1.313d0*log10_cr(M1/Msun/30) &
                  - 1.226d0*log10_cr(vinf_div_vesc/2d0) &
                  + 0.933d0*log10_cr(T1/4d4) &
                  - 10.92d0*pow2(log10_cr(T1/4d4)) &
                  + 0.85d0*log10_cr(Z/Zsolar)
               w1 = exp10_cr(logMdot)
               w = w1
            else if (T1 < 22500d0) then
        vinf_div_vesc = 1.3d0 ! this is the cool side galactic value   
        vinf_div_vesc = vinf_div_vesc*pow_cr(Z/Zsolar,0.13d0) ! corrected for Z                                                
               logMdot = &
                  - 6.688d0 &
                  + 2.210d0*log10_cr(L1/Lsun/1d5) &
                  - 1.339d0*log10_cr(M1/Msun/30) &
                  - 1.601d0*log10_cr(vinf_div_vesc/2d0) &
                  + 1.07d0*log10_cr(T1/2d4) &
                  + 0.85d0*log10_cr(Z/Zsolar)
               w2 = exp10_cr(logMdot)
               w = w2
            else ! use Vink et al 2001, eqns 14 and 15 to set "jump" temperature 
               Teff_jump = 1d3*(61.2d0 + 2.59d0*(-13.636d0 + 0.889d0*log10_cr(Z/Zsolar)))                     
               dT = 100d0
               print *, 'Teff_jump[K]', Teff_jump
!     first evaluate the mass loss rate one would have if T = 27500, all rest held constant
               vinf_div_vesc = 2.6d0 ! this is the hot side galactic value
               vinf_div_vesc = vinf_div_vesc*pow_cr(Z/Zsolar,0.13d0) ! corrected for Z                                                
               w_h =  &
                  - 6.697d0 &
                  + 2.194d0*log10_cr(L1/Lsun/1d5) &
                  - 1.313d0*log10_cr(M1/Msun/30) &
                  - 1.226d0*log10_cr(vinf_div_vesc/2d0) &
                  + 0.933d0*log10_cr(27500/4d4) &
                  - 10.92d0*pow2(log10_cr(27500/4d4)) &
                  + 0.85d0*log10_cr(Z/Zsolar)
               w_h = exp10_cr(w_h)
               !then evaluate the mass loss rate one would have if T= 22500, all rest held constant
               vinf_div_vesc = 1.3d0 ! this is the cool side galactic value                                                                                  
               vinf_div_vesc = vinf_div_vesc*pow_cr(Z/Zsolar,0.13d0) ! corrected for Z                                                                       
               w_c = &
                  - 6.688d0 &
                  + 2.210d0*log10_cr(L1/Lsun/1d5) &
                  - 1.339d0*log10_cr(M1/Msun/30) &
                  - 1.601d0*log10_cr(vinf_div_vesc/2d0) &
                  + 1.07d0*log10_cr(22500/2d4) &
                  + 0.85d0*log10_cr(Z/Zsolar)
               w_c = exp10_cr(w_c)
               !then interpolate between this two values
               w = ((w_h - w_c)/2)*TANH((T1-Teff_jump)/(1d0*dT)) &
                   +(w_c+w_h)/2
               !print *, 'USING TANH(',(T1-Teff_jump)/(10d0*dT), &
               !       ')=',&
               !       TANH(T1-Teff_jump)/(10d0*dT), 'w_c=', &
               !       w_c,&
               !       'w_h=', w_h, 'w=',w
            end if

           end subroutine eval_tanh_intrp_vink_wind


      end subroutine  Dutch_tanh_intrp_wind

      subroutine extras_controls(s, ierr)
         type (star_info), pointer :: s
         integer, intent(out) :: ierr
         ierr = 0
         
         ! this is the place to set any procedure pointers you want to change
         ! e.g., other_wind, other_mixing, other_energy  (see star_data.inc)
         s% other_wind =>  Dutch_tanh_intrp_wind       
      end subroutine extras_controls
      
      
      integer function extras_startup(s, id, restart, ierr)
         type (star_info), pointer :: s
         integer, intent(in) :: id
         logical, intent(in) :: restart
         integer, intent(out) :: ierr
         ierr = 0
         extras_startup = 0
		 call system_clock(time0,clock_rate)
         if (.not. restart) then
            call alloc_extra_info(s)
         else ! it is a restart
            call unpack_extra_info(s)
         end if
      end function extras_startup
      

      ! returns either keep_going, retry, backup, or terminate.
      integer function extras_check_model(s, id, id_extra)
         use crlibm_lib         
         type (star_info), pointer :: s
         integer, intent(in) :: id, id_extra
         real(dp) :: mdot, mdot_old
         extras_check_model = keep_going         
         if (.false. .and. s% star_mass_h1 < 0.35d0) then
            ! stop when star hydrogen mass drops to specified level
            extras_check_model = terminate
            write(*, *) 'have reached desired hydrogen mass'
            return
         end if

         if (extras_check_model == terminate) s% termination_code = t_extras_check_model
      end function extras_check_model


      integer function how_many_extra_history_columns(s, id, id_extra)
         type (star_info), pointer :: s
         integer, intent(in) :: id, id_extra
	 integer ::  n_cols
         n_cols = 0
         how_many_extra_history_columns = 0

      end function how_many_extra_history_columns
      
      
      subroutine data_for_extra_history_columns(s, id, id_extra, n, names, vals, ierr)
         type (star_info), pointer :: s
         integer, intent(in) :: id, id_extra, n
         character (len=maxlen_history_column_name) :: names(n)
         real(dp) :: vals(n)
         integer, intent(out) :: ierr
         ierr = 0
      end subroutine data_for_extra_history_columns

      
      integer function how_many_extra_profile_columns(s, id, id_extra)
         type (star_info), pointer :: s
         integer, intent(in) :: id, id_extra
         how_many_extra_profile_columns = 1
      end function how_many_extra_profile_columns
      
      
      subroutine data_for_extra_profile_columns(s, id, id_extra, n, nz, names, vals, ierr)
         type (star_info), pointer :: s
         integer, intent(in) :: id, id_extra, n, nz
         character (len=maxlen_profile_column_name) :: names(n)
         real(dp) :: vals(nz,n)
         integer, intent(out) :: ierr
         integer :: k
         ierr = 0
         
         !note: do NOT add these names to profile_columns.list
         ! the profile_columns.list is only for the built-in profile column options.
         ! it must not include the new column names you are adding here.

         ! here is an example for adding a profile column
         if (n /= 1) stop 'data_for_extra_profile_columns'
         !names(1) = 'beta'
         !do k = 1, nz
         !   vals(k,1) = s% Pgas(k)/s% P(k)
         !end do
         
      end subroutine data_for_extra_profile_columns
      

      ! returns either keep_going or terminate.
      ! note: cannot request retry or backup; extras_check_model can do that.
      integer function extras_finish_step(s, id, id_extra)
         use crlibm_lib
         type (star_info), pointer :: s
         integer, intent(in) :: id, id_extra
         integer :: ierr, k
         real(dp) :: time, mdot_timescale, sound_time_env, rad_diff_env, rad_process_timescale, random_walk_num_step
         real(dp) :: env_R, env_mfp, R_core, deltaR, env_M, M_core, tmp
         real(dp) :: mean_rho, mean_op
         
         call system_clock(time1,clock_rate)
         time = dble(time1-time0)/ clock_rate / 60 ! elapsed time in minutes
         extras_finish_step = keep_going
         call store_extra_info(s)

         !Stop the run after a specified amount of time
         if (time .GT. 300) then !300 min = 5h00 !715 min = 11h:55min
            call output(id, ierr) !to save a photo
            extras_finish_step = terminate 
            print *, 'Terminated because maximum time reached'
         end if
            call get_star_ptr(id,s,ierr)
            s% lxtra1 = .false.
            if (s% xa(8,s%nz) .LT. 0.04) then
               if (s%xa(11, s%nz) .GT. 0.01) then !O depletion: X_o16 < 0.04 in the center and si28 has been produced
                  extras_finish_step = keep_going!terminate
                  s%lxtra1 = .true.
                  if ((s%lxtra1_old .eqv. .false.) .AND. (s% lxtra1 .eqv. .true.)) then
                     s% need_to_save_profiles_now = .true.
                     s% save_profiles_model_priority = 10
                     call output(id, ierr) !to save a photo
                  end if
               end if
            end if
            
          
       !control that the next timestep is smaller than the relevant timescales

       
         rad_process_timescale = 1d-8 !in sec 

         ! find the envelope boundary
         k = s% nz
         do while (s% m(k) / Msun .LT. s% mass_conv_core) !in Msun
            k = k-1
         end do
         R_core = s% r(k) / Rsun !in Rsun
         M_core = s% m(k) / Msun
         R_core = max(R_core, s% he_core_radius) !in Rsun
         M_core = max(M_core, s% he_core_mass) !in Msun
         !he_core_radius is the radius where H drops below 0.01
         env_R = s% r(1) / Rsun - R_core !in Rsun
         env_M = s% mstar /Msun - M_core !in Msun
         env_M = env_M * Msun !in grams
         if (env_R /= 0) then
            k = s% nz
            do while ((s% r(k) / Rsun .LT. R_core))
               k = k-1 !move outward
            end do              !at the end of the loop k is the index of the first cell of the envelope
            !print *, s% r(k)
            mean_rho = 0
            mean_op = 0
            tmp = 0
            do while (k .GE. 1)
               if (k == s% nz) then
                  deltaR = s%r(k) !in cm
                  tmp = tmp + deltaR / s% csound(k) !in sec
                  ! evaluate the mean opacity and density in the envelope weighting by the radius
                  mean_op = mean_op + s% opacity(k) * s% r(k)
                  mean_rho = mean_rho + s% rho(k) * s% r(k)
               else !means we are not in the central cell
                  deltaR = s% r(k) / Rsun - s% r(k+1) / Rsun
                  deltaR = deltaR * Rsun !in cm
                   ! evaluate the mean opacity and density in the envelope weighting by the radius
                  tmp = tmp + deltaR / s% csound(k) !in sec
                  mean_op = mean_op + s% opacity(k) * ((s% r(k)/ Rsun &
                            - s% r(k+1) / Rsun )) * Rsun
                  mean_rho = mean_rho + s% rho(k) *  ((s% r(k)/ Rsun &
                            - s% r(k+1) / Rsun )) * Rsun
               end if
               k = k-1
             end do
            mean_rho = mean_rho / (env_R * Rsun) !normalization
            mean_op = mean_op / (env_R * Rsun) !normalization
            env_R = env_R * Rsun !in cm
            env_mfp = 1d0 / (mean_rho * mean_op)  !in cm
            random_walk_num_step = (env_R/env_mfp)**2 !pure number
            rad_diff_env = rad_process_timescale * random_walk_num_step !in sec
            sound_time_env = tmp !in sec
         else 
            rad_diff_env = 1d99
            sound_time_env = 1d99
         end if

         mdot_timescale = s% mstar / (1d0 * abs(s% mstar_dot)) ! in sec
         s% dt_next = min(s% dt_next / secyer , s% kh_timescale, &
                          mdot_timescale / secyer, & 
                          s% nuc_timescale, &
                          s% chem_timescale , &
                          rad_diff_env / secyer) 
         !print *, s% dt_next
         s% dt_next = s% dt_next * secyer
         if (min(s% dt_next / secyer , s% kh_timescale, &
                          mdot_timescale / secyer, & 
                          s% nuc_timescale, &
                          s% chem_timescale , &
                          rad_diff_env / secyer) & 
                          == s% kh_timescale ) then
            print *, "dt_next limited by kh_timescale"
         else if (min(s% dt_next / secyer , s% kh_timescale, &
                          mdot_timescale / secyer, & 
                          s% nuc_timescale, &
                          s% chem_timescale , &
                          rad_diff_env / secyer) &
                          == mdot_timescale / secyer )  then
             print *, "dt_next limited by mdot_timescale"
          else if (min(s% dt_next / secyer , s% kh_timescale, &
                          mdot_timescale / secyer, & 
                          s% nuc_timescale, &
                          s% chem_timescale , &
                          rad_diff_env / secyer) &
                          ==  s% dynamic_timescale / secyer ) then
             print *, "dt_next limited by dynamic_timescale"
          else if (min(s% dt_next / secyer , s% kh_timescale, &
                          mdot_timescale / secyer, & 
                          s% nuc_timescale, &
                          s% chem_timescale , &
                          rad_diff_env / secyer) &
                          == s% nuc_timescale ) then
             print *, "dt_next limited by nuc_timescale"
          else if (min(s% dt_next / secyer , s% kh_timescale, &
                          mdot_timescale / secyer, & 
                          s% nuc_timescale, &
                          s% chem_timescale , &
                          rad_diff_env / secyer) &
                          == s% chem_timescale * secyer) then
             print *, "dt_next limited by chem_timescale"
          else if (min(s% dt_next / secyer , s% kh_timescale, &
                          mdot_timescale / secyer, & 
                          s% nuc_timescale, &
                          s% chem_timescale , &
                          rad_diff_env / secyer) &
                          == rad_diff_env /  secyer) then
             print *, "dt_next limited by rad_diff_env"
          else if (min(s% dt_next / secyer, s% kh_timescale, &
                          mdot_timescale / secyer, & 
                          s% nuc_timescale , &
                          s% chem_timescale, &
                          rad_diff_env / secyer) &
                          == s% dt_next / secyer ) then
             !print no custom timestep limit
          end if

          contains
	
         subroutine output(id, ierr)
    	    ! use star_utils, only: get_name_for_restart_file
         use crlibm_lib
       	  interface
       	     subroutine save_restart_info(iounit, id, ierr)
       	        integer, intent(in) :: iounit
       	        integer, intent(in) :: id
       	        integer, intent(out) :: ierr
       	     end subroutine save_restart_info
       	  end interface
          integer, intent(in) :: id
          integer, intent(out) :: ierr     
          
          character (len=256) :: filename, num_str, fstring
          type (star_info), pointer :: s
          integer :: num_digits
 
          call get_star_ptr(id, s, ierr)
          if (ierr /= 0) return
          num_str = 'photo_5h00min'          
          if (ierr /= 0) return    
          if ((s% xa(8,s%nz) .LT. 0.04) .AND. (s%xa(11, s%nz) .GT. 0.01)) then !O depletion: X_o16 < 0.04 in the center and si28 has been produced
             num_str = 'photo_O_depl'
          end if

          filename = trim(s% photo_directory) // '/' // trim(num_str) 
          call output_to_file(filename, id, ierr) 

       end subroutine output


      subroutine output_to_file(filename, id, ierr)
         use utils_lib, only:alloc_iounit, free_iounit
         use model_out, only:output_star_model
         character (len=*) :: filename
         integer, intent(in) :: id
         integer, intent(out) :: ierr

         integer :: iounit, k
         type (star_info), pointer :: s
         character(len=256) :: iomsg

         include 'formats'

         call get_star_ptr(id, s, ierr)
         if (ierr /= 0) return

         iounit = alloc_iounit(ierr); if (ierr /= 0) return
         open(iounit, file=trim(filename), action='write', &
         status='replace', iostat=ierr, iomsg=iomsg, form='unformatted')
         if (ierr == 0) then
            s% most_recent_photo_name = trim(filename)
            call output_star_model(s, iounit)
            close(iounit)
         else
            write(*,*) trim(iomsg)
         endif
         call free_iounit(iounit)
         
      end subroutine output_to_file

         ! to save a profile, 
            ! s% need_to_save_profiles_now = .true.
         ! to update the star log,
            ! s% need_to_update_history_now = .true.

         ! see extras_check_model for information about custom termination codes
         ! by default, indicate where (in the code) MESA terminated
         !if (extras_finish_step == terminate) s% termination_code = t_extras_finish_step
      end function extras_finish_step
      
      
      subroutine extras_after_evolve(s, id, id_extra, ierr)
         type (star_info), pointer :: s
         integer, intent(in) :: id, id_extra
         integer, intent(out) :: ierr
         ierr = 0
      end subroutine extras_after_evolve
      
      
      ! routines for saving and restoring extra data so can do restarts
         
         ! put these defs at the top and delete from the following routines
         !integer, parameter :: extra_info_alloc = 1
         !integer, parameter :: extra_info_get = 2
         !integer, parameter :: extra_info_put = 3
      
      
      subroutine alloc_extra_info(s)
         integer, parameter :: extra_info_alloc = 1
         type (star_info), pointer :: s
         call move_extra_info(s,extra_info_alloc)
      end subroutine alloc_extra_info
      
      
      subroutine unpack_extra_info(s)
         integer, parameter :: extra_info_get = 2
         type (star_info), pointer :: s
         call move_extra_info(s,extra_info_get)
      end subroutine unpack_extra_info
      
      
      subroutine store_extra_info(s)
         integer, parameter :: extra_info_put = 3
         type (star_info), pointer :: s
         call move_extra_info(s,extra_info_put)
      end subroutine store_extra_info
      
      
      subroutine move_extra_info(s,op)
         integer, parameter :: extra_info_alloc = 1
         integer, parameter :: extra_info_get = 2
         integer, parameter :: extra_info_put = 3
         type (star_info), pointer :: s
         integer, intent(in) :: op
         
         integer :: i, j, num_ints, num_dbls, ierr
         
         i = 0
         ! call move_int or move_flg    
         num_ints = i
         
         i = 0
         ! call move_dbl       
         
         num_dbls = i
         
         if (op /= extra_info_alloc) return
         if (num_ints == 0 .and. num_dbls == 0) return
         
         ierr = 0
         call star_alloc_extras(s% id, num_ints, num_dbls, ierr)
         if (ierr /= 0) then
            write(*,*) 'failed in star_alloc_extras'
            write(*,*) 'alloc_extras num_ints', num_ints
            write(*,*) 'alloc_extras num_dbls', num_dbls
            stop 1
         end if
         
         contains
         
         subroutine move_dbl(dbl)
            real(dp) :: dbl
            i = i+1
            select case (op)
            case (extra_info_get)
               dbl = s% extra_work(i)
            case (extra_info_put)
               s% extra_work(i) = dbl
            end select
         end subroutine move_dbl
         
         subroutine move_int(int)
            integer :: int
            i = i+1
            select case (op)
            case (extra_info_get)
               int = s% extra_iwork(i)
            case (extra_info_put)
               s% extra_iwork(i) = int
            end select
         end subroutine move_int
         
         subroutine move_flg(flg)
            logical :: flg
            i = i+1
            select case (op)
            case (extra_info_get)
               flg = (s% extra_iwork(i) /= 0)
            case (extra_info_put)
               if (flg) then
                  s% extra_iwork(i) = 1
               else
                  s% extra_iwork(i) = 0
               end if
            end select
         end subroutine move_flg
      
      end subroutine move_extra_info

      end module run_star_extras
      
